@@ -1,10 +1,11 @@
-import os
 import json
+import pathlib
+import toolviper.utils.logger as logger
+
 import numpy as np
 
 from casacore import tables
-
-from astrohack._utils._logger._astrohack_logger import _get_astrohack_logger
+from rich.console import Console
 
 from astrohack.mds import AstrohackImageFile
 from astrohack.mds import AstrohackHologFile
@@ -13,18 +14,21 @@ from astrohack.mds import AstrohackPointFile
 from astrohack.mds import AstrohackLocitFile
 from astrohack.mds import AstrohackPositionFile
 
+from astrohack.utils.text import print_array
 
-from astrohack._utils._dio import _print_array
+from typing import Union, List, NewType, Dict, Any, NoReturn
+
+JSON = NewType("JSON", Dict[str, Any])
 
 
-def open_holog(file):
+def open_holog(file: str) -> Union[AstrohackHologFile, None]:
     """ Open holog file and return instance of the holog data object. Object includes summary function to list\
      available dictionary keys.
 
     :param file: Path to holog file.
     :type file: str
   
-    :return: Holography holog object.
+    :return: Holography holog object; None if file not found.
     :rtype: AstrohackHologFile
 
     .. _Description:
@@ -50,25 +54,23 @@ def open_holog(file):
             }
     """
 
-    logger = _get_astrohack_logger()
-
     _data_file = AstrohackHologFile(file=file)
 
-    if _data_file._open():
+    if _data_file.open():
         return _data_file
 
     else:
-        logger.error(f"Error opening holgraphy file: {file}")
+        return None
 
 
-def open_image(file):
+def open_image(file: str) -> Union[AstrohackImageFile, None]:
     """ Open image file and return instance of the image data object. Object includes summary function to list \
     available dictionary keys.
 
     :param file: Path to image file.
     :type file: str
   
-    :return: Holography image object.
+    :return: Holography image object; None if file not found.
     :rtype: AstrohackImageFile
 
     .. _Description:
@@ -91,25 +93,23 @@ def open_image(file):
 
     """
 
-    logger = _get_astrohack_logger()
-
     _data_file = AstrohackImageFile(file=file)
 
-    if _data_file._open():
+    if _data_file.open():
         return _data_file
 
     else:
-        logger.error(f"Error opening holgraphy image file: {file}")
+        return None
 
 
-def open_panel(file):
+def open_panel(file: str) -> Union[AstrohackPanelFile, None]:
     """ Open panel file and return instance of the panel data object. Object includes summary function to list \
     available dictionary keys.
 
     :param file: Path ot panel file.
     :type file: str
 
-    :return: Holography panel object.
+    :return: Holography panel object; None if file not found.
     :rtype: AstrohackPanelFile
 
     .. _Description:
@@ -132,25 +132,23 @@ def open_panel(file):
 
     """
 
-    logger = _get_astrohack_logger()
-
     _data_file = AstrohackPanelFile(file=file)
 
-    if _data_file._open():
+    if _data_file.open():
         return _data_file
 
     else:
-        logger.error(f"Error opening holgraphy panel file: {file}")
+        return None
 
 
-def open_locit(file):
+def open_locit(file: str) -> Union[AstrohackLocitFile, None]:
     """ Open locit file and return instance of the locit data object. Object includes summary function to list \
     available dictionary keys.
 
     :param file: Path of locit file.
     :type file: str
 
-    :return: locit object.
+    :return: locit object; None if file not found.
     :rtype: AstrohackLocitFile
 
     .. _Description:
@@ -173,25 +171,23 @@ def open_locit(file):
 
     """
 
-    logger = _get_astrohack_logger()
-
     _data_file = AstrohackLocitFile(file=file)
 
-    if _data_file._open():
+    if _data_file.open():
         return _data_file
 
     else:
-        logger.error(f"Error opening holgraphy locit file: {file}")
+        return
 
 
-def open_position(file):
+def open_position(file: str) -> Union[AstrohackPositionFile, None]:
     """ Open position file and return instance of the position data object. Object includes summary function to list \
     available dictionary keys.
 
     :param file: Path of position file.
     :type file: str
 
-    :return: position object.
+    :return: position object; None if file does not exist.
     :rtype: AstrohackPositionFile
 
     .. _Description:
@@ -214,25 +210,23 @@ def open_position(file):
 
     """
 
-    logger = _get_astrohack_logger()
-
     _data_file = AstrohackPositionFile(file=file)
 
-    if _data_file._open():
+    if _data_file.open():
         return _data_file
 
     else:
-        logger.error(f"Error opening holgraphy position file: {file}")
+        return None
 
 
-def open_pointing(file):
+def open_pointing(file: str) -> Union[AstrohackPointFile, None]:
     """ Open pointing file and return instance of the pointing data object. Object includes summary function to list\
      available dictionary keys.
 
     :param file: Path to pointing file.
     :type file: str
   
-    :return: Holography pointing object.
+    :return: Holography pointing object; None if file does not exist.
     :rtype: AstrohackPointFile
 
     .. _Description:
@@ -252,25 +246,23 @@ def open_pointing(file):
 
     """
 
-    logger = _get_astrohack_logger()
-
     _data_file = AstrohackPointFile(file=file)
 
-    if _data_file._open():
+    if _data_file.open():
         return _data_file
 
     else:
-        logger.error(f"Error opening holography pointing file: {file}")
+        return None
 
 
-def fix_pointing_table(ms_name, reference_antenna):    
+def fix_pointing_table(ms_name: str, reference_antenna: List[str]) -> None:
     """ Fix pointing table for a user defined subset of reference antennas.
 
     :param ms_name: Measurement set name.
-    :type file: str
+    :type ms_name: str
 
     :param reference_antenna: List of reference antennas.
-    :type file: list
+    :type reference_antenna: list
   
     .. _Description:
 
@@ -288,17 +280,23 @@ def fix_pointing_table(ms_name, reference_antenna):
 
   """
 
-    ms_table = "/".join((ms_name, 'ANTENNA'))
+    path = pathlib.Path(ms_name)
+    ms_name_fullpath = str(path.absolute().resolve())
 
-    query = 'select NAME from {table}'.format(table=ms_table)
+    if not path.exists():
+        logger.error("Error finding file: {file}".format(file=ms_name_fullpath))
+
+    ms_table = "/".join((ms_name_fullpath, 'ANTENNA'))
+
+    query = 'select NAME from "{table}"'.format(table=ms_table)
 
     ant_names = np.array(tables.taql(query).getcol('NAME'))
-    
+
     ant_id = np.arange(len(ant_names))
 
     query_ant = np.searchsorted(ant_names, reference_antenna)
 
-    ms_table = "/".join((ms_name, 'POINTING'))
+    ms_table = "/".join((ms_name_fullpath, 'POINTING'))
 
     ant_list = " or ".join(["ANTENNA_ID=={ant}".format(ant=ant) for ant in query_ant])
 
@@ -307,7 +305,7 @@ def fix_pointing_table(ms_name, reference_antenna):
 
     tables.taql(update)
 
-    ms_table = "/".join((ms_name, "HISTORY"))
+    ms_table = "/".join((ms_name_fullpath, "HISTORY"))
     tb = tables.table(ms_table, readonly=False)
 
     message = tb.getcol("MESSAGE")
@@ -318,54 +316,65 @@ def fix_pointing_table(ms_name, reference_antenna):
         tb.putcol(columnname="MESSAGE", value='pnt_tbl:fixed', startrow=length)
 
 
-def print_json(obj, indent=6, columns=7):
-    """ Print formatted JSON dictionary
+def print_json(
+        obj: JSON,
+        indent: int = 6,
+        columns: int = 7
+) -> None:
+    """ Print formatted JSON dictionary (** Deprecated by Console **)
 
     :param obj: JSON object
     :type obj: JSON
+
     :param indent: Indent to be used in JSON dictionary., defaults to 6
     :type indent: int, optional
+
     :param columns: Columns used to reshape the antenna list., defaults to 7
     :type columns: int, optional
     """
-  
+    import toolviper.utils.console as console
+
+    color = console.Colorize()
+
     if isinstance(obj, np.ndarray):
         obj = list(obj)
 
     if isinstance(obj, list):
         if indent > 3:
-            list_indent = indent-3
+            list_indent = indent - 3
         else:
             list_indent = 0
 
         print("{open}".format(open="[").rjust(list_indent, ' '))
-        _print_array(obj, columns=columns, indent=indent + 1)
+        print_array(obj, columns=columns, indent=indent + 1)
         print("{close}".format(close="]").rjust(list_indent, ' '))
 
     else:
         for key, value in obj.items():
-            key_str="{key}{open}".format(key=key, open=":{")
+            key_str = "{key}{open}".format(key=key, open=":{")
             print("{key}".format(key=key_str).rjust(indent, ' '))
-            print_json(value, indent+4, columns=columns)
-            print("{close}".format(close="}").rjust(indent-4, ' '))
+            print_json(value, indent + 4, columns=columns)
+            print("{close}".format(close="}").rjust(indent - 4, ' '))
 
 
-def inspect_holog_obs_dict(file='.holog_obs_dict.json', style='static', indent=6, columns=7):
+def inspect_holog_obs_dict(
+        file: Union[str, JSON] = '.holog_obs_dict.json',
+        style: str = 'static'
+) -> Union[NoReturn, JSON]:
     """ Print formatted holography observation dictionary
 
     :param file: Input file, can be either JSON file or string., defaults to '.holog_obs_dict.json'
     :type file: str | JSON, optional
-    :param style: Print style of JSON dictionary. This can be static, formatted generalized print out or dynamic, prints a collapsible formatted dictionary, defaults to static
+
+    :param style: Print style of JSON dictionary. This can be static, formatted generalized print out or dynamic, \
+    prints a collapsible formatted dictionary, defaults to static
     :type style: str, optional
-    :param indent: Indent to be used in JSON dictionary., defaults to 6
-    :type indent: int, optional
-    :param columns: Columns used to reshape the antenna list., defaults to 7
-    :type columns: int, optional
 
     .. _Description:
 
     **Example Usage**
-    The `inspect_holog_obs_dict` loads a holography observation dict either from disk or from memory (as an return value from `generate_holog_obs_dict`) and displays it in a more readable way like JSON.stringify() in javascript.
+    The `inspect_holog_obs_dict` loads a holography observation dict either from disk or from memory (as an return \
+    value from `generate_holog_obs_dict`) and displays it in a more readable way like JSON.stringify() in javascript.
 
     .. parsed-literal::
         import astrohack
@@ -393,24 +402,29 @@ def inspect_holog_obs_dict(file='.holog_obs_dict.json', style='static', indent=6
             }
         } 
     """
-    logger = _get_astrohack_logger()
 
     if not isinstance(file, dict):
         try:
             with open(file) as json_file:
                 json_object = json.load(json_file)
-            
+
+        except IsADirectoryError:
+            try:
+                with open(file+'/holog_obs_dict.json') as json_file:
+                    json_object = json.load(json_file)
+            except FileNotFoundError:
+                logger.error("holog observations dictionary not found: {file}".format(file=file))
         except FileNotFoundError:
             logger.error("holog observations dictionary not found: {file}".format(file=file))
-    
+
     else:
         json_object = file
 
-    if style == 'dynamic': 
+    if style == 'dynamic':
         from IPython.display import JSON
 
         return JSON(json_object)
-        
-    
+
     else:
-        print_json(obj=json_object, indent=indent, columns=columns)
+        console = Console()
+        console.log(json_object, log_locals=False)
