@@ -1,46 +1,270 @@
 import copy
-import inspect
 import json
 import os
+import pathlib
+import pickle
+import shutil
+import math
+import multiprocessing
 
+import toolviper.utils.parameter
 import dask
 import astrohack
+import psutil
+
 import numpy as np
+import toolviper.utils.logger as logger
 
-from astrohack._utils._constants import pol_str
-from astrohack._utils._conversion import _convert_ant_name_to_id
-from astrohack._utils._dio import _check_if_file_exists
-from astrohack._utils._dio import _check_if_file_will_be_overwritten
-from astrohack._utils._dio import _load_holog_file
-from astrohack._utils._dio import _load_point_file
-from astrohack._utils._dio import _write_meta_data
-from astrohack._utils._extract_holog import _create_holog_meta_data
-from astrohack._utils._extract_holog import _create_holog_obs_dict
-from astrohack._utils._extract_holog import _extract_holog_chunk
-from astrohack._utils._logger._astrohack_logger import _get_astrohack_logger
-from astrohack._utils._param_utils._check_parms import _check_parms, _parm_check_passed
-from astrohack._utils._tools import NumpyEncoder
-from astrohack._utils._tools import _remove_suffix
-from astrohack.mds import AstrohackHologFile
-from astrohack.mds import AstrohackPointFile
 from astropy.time import Time
 from casacore import tables as ctables
+from rich.console import Console
+from rich.table import Table
 
-CURRENT_FUNCTION = 0
+from astrohack.utils.constants import pol_str
+
+from astrohack.utils.file import overwrite_file
+from astrohack.utils.file import load_holog_file
+from astrohack.utils.file import load_point_file
+from astrohack.utils.data import write_meta_data
+from astrohack.core.extract_holog import create_holog_meta_data
+from astrohack.core.extract_holog import create_holog_obs_dict
+from astrohack.core.extract_holog import process_extract_holog_chunk
+from astrohack.utils.tools import get_valid_state_ids
+from astrohack.utils.text import get_default_file_name
+from astrohack.utils.text import NumpyEncoder
+from astrohack.mds import AstrohackHologFile
+from astrohack.mds import AstrohackPointFile
+from astrohack.extract_pointing import extract_pointing
+
+from typing import Union, List, NewType, Dict, Any, Tuple
+
+JSON = NewType("JSON", Dict[str, Any])
+KWARGS = NewType("KWARGS", Union[Dict[str, str], Dict[str, int]])
 
 
+class HologObsDict(dict):
+    """
+      ddi --> map --> ant, scan
+                      |
+                      o--> map: [reference, ...]
+    """
+
+    def __init__(self, obj: JSON = None):
+        if obj is None:
+            super().__init__()
+        else:
+            super().__init__(obj)
+
+    def __getitem__(self, key: str):
+        return super().__getitem__(key)
+
+    def __setitem__(self, key: str, value: Any):
+        return super().__setitem__(key, value)
+
+    @classmethod
+    def from_file(cls, filepath):
+        if filepath.endswith(".holog.zarr"):
+            filepath = str(pathlib.Path(filepath).resolve().joinpath("holog_obs_dict.json"))
+
+        try:
+            with open(filepath, "r") as file:
+                obj = json.load(file)
+
+                return HologObsDict(obj)
+
+        except FileNotFoundError:
+            logger.error(f"File {filepath} not found")
+
+    def print(self, style: str = "static"):
+        if style == "dynamic":
+            return astrohack.dio.inspect_holog_obs_dict(self, style="dynamic")
+
+        else:
+            return astrohack.dio.inspect_holog_obs_dict(self, style="static")
+
+    def select(self, key: str, value: any, inplace: bool = False, **kwargs: KWARGS) -> object:
+
+        if inplace:
+            obs_dict = self
+
+        else:
+            obs_dict = HologObsDict(copy.deepcopy(self))
+
+        if key == "ddi":
+            return self._select_ddi(value, obs_dict=obs_dict)
+
+        elif key == "map":
+            return self._select_map(value, obs_dict=obs_dict)
+
+        elif key == "antenna":
+            return self._select_antenna(value, obs_dict=obs_dict)
+
+        elif key == "scan":
+            return self._select_scan(value, obs_dict=obs_dict)
+
+        elif key == "baseline":
+            if "reference" in kwargs.keys():
+                return self._select_baseline(
+                    value,
+                    n_baselines=None,
+                    reference=kwargs["reference"],
+                    obs_dict=obs_dict
+                )
+
+            elif "n_baselines" in kwargs.keys():
+                return self._select_baseline(
+                    value,
+                    n_baselines=kwargs["n_baselines"],
+                    reference=None,
+                    obs_dict=obs_dict
+                )
+
+            else:
+                logger.error("Must specify a list of reference antennas for this option.")
+        else:
+            logger.error("Valid key not found: {key}".format(key=key))
+            return {}
+
+    @staticmethod
+    def get_nearest_baselines(antenna: str, n_baselines: int = None, path_to_matrix: str = None) -> object:
+        import pandas as pd
+
+        if path_to_matrix is None:
+            path_to_matrix = str(pathlib.Path.cwd().joinpath(".baseline_distance_matrix.csv"))
+
+        if not pathlib.Path(path_to_matrix).exists():
+            logger.error("Unable to find baseline distance matrix in: {path}".format(path=path_to_matrix))
+
+        df_matrix = pd.read_csv(path_to_matrix, sep="\t", index_col=0)
+
+        # Skip the first index because it is a self distance
+        if n_baselines is None:
+            return df_matrix[antenna].sort_values(ascending=True).index[1:].values.tolist()
+
+        return df_matrix[antenna].sort_values(ascending=True).index[1:n_baselines].values.tolist()
+
+    @staticmethod
+    def _select_ddi(value: Union[int, List[int]], obs_dict: object) -> object:
+        convert = lambda x: "ddi_" + str(x)
+
+        if not isinstance(value, list):
+            value = [value]
+
+        value = list(map(convert, value))
+        ddi_list = list(obs_dict.keys())
+
+        for ddi in ddi_list:
+            if ddi not in value:
+                obs_dict.pop(ddi)
+
+        return obs_dict
+
+    @staticmethod
+    def _select_map(value: Union[int, List[int]], obs_dict: object) -> object:
+        convert = lambda x: "map_" + str(x)
+
+        if not isinstance(value, list):
+            value = [value]
+
+        value = list(map(convert, value))
+        ddi_list = list(obs_dict.keys())
+
+        for ddi in ddi_list:
+            map_list = list(obs_dict[ddi].keys())
+            for mp in map_list:
+                if mp not in value:
+                    obs_dict[ddi].pop(mp)
+
+        return obs_dict
+
+    @staticmethod
+    def _select_antenna(value: Union[str, List[str]], obs_dict: object) -> object:
+        if not isinstance(value, list):
+            value = [value]
+
+        ddi_list = list(obs_dict.keys())
+
+        for ddi in ddi_list:
+            map_list = list(obs_dict[ddi].keys())
+            for mp in map_list:
+                ant_list = list(obs_dict[ddi][mp]["ant"].keys())
+                for ant in ant_list:
+                    if ant not in value:
+                        obs_dict[ddi][mp]["ant"].pop(ant)
+
+        return obs_dict
+
+    @staticmethod
+    def _select_scan(value: Union[int, List[int]], obs_dict: object) -> object:
+        if not isinstance(value, list):
+            value = [value]
+
+        ddi_list = list(obs_dict.keys())
+
+        for ddi in ddi_list:
+            map_list = list(obs_dict[ddi].keys())
+            for mp in map_list:
+                obs_dict[ddi][mp]["scans"] = value
+
+        return obs_dict
+
+    @staticmethod
+    def _select_baseline(
+            value: str,
+            n_baselines: int,
+            obs_dict: object,
+            reference: Union[str, List[int]] = None
+    ) -> object:
+        if reference is not None:
+            if not isinstance(reference, list):
+                reference = [reference]
+
+        ddi_list = list(obs_dict.keys())
+
+        for ddi in ddi_list:
+            map_list = list(obs_dict[ddi].keys())
+            for mp in map_list:
+                ant_list = list(obs_dict[ddi][mp]["ant"].keys())
+                for ant in ant_list:
+                    if ant not in value:
+                        obs_dict[ddi][mp]["ant"].pop(ant)
+                        continue
+
+                    if reference is None and n_baselines is not None:
+                        reference_antennas = obs_dict[ddi][mp]["ant"][ant]
+
+                        if n_baselines > len(reference_antennas):
+                            n_baselines = len(reference_antennas)
+
+                        sorted_antennas = np.array(obs_dict.get_nearest_baselines(antenna=ant))
+
+                        values, i, j = np.intersect1d(reference_antennas, sorted_antennas, return_indices=True)
+                        index = np.sort(j)
+
+                        obs_dict[ddi][mp]["ant"][ant] = sorted_antennas[index][:n_baselines]
+
+                    else:
+                        obs_dict[ddi][mp]["ant"][ant] = reference
+
+        return obs_dict
+
+
+@toolviper.utils.parameter.validate(
+    add_data_type=HologObsDict
+)
 def extract_holog(
-        ms_name,
-        point_name,
-        holog_name=None,
-        holog_obs_dict=None,
-        ddi='all',
-        baseline_average_distance='all',
-        baseline_average_nearest='all',
-        data_column="CORRECTED_DATA",
-        parallel=False,
-        overwrite=False,
-):
+        ms_name: str,
+        point_name: str,
+        holog_name: str = None,
+        holog_obs_dict: HologObsDict = None,
+        ddi: Union[int, List[int], str] = 'all',
+        baseline_average_distance: Union[float, str] = 'all',
+        baseline_average_nearest: Union[float, str] = 'all',
+        data_column: str = "CORRECTED_DATA",
+        time_smoothing_interval: float = None,
+        parallel: bool = False,
+        overwrite: bool = False,
+) -> Union[AstrohackHologFile, None]:
     """
     Extract holography and optionally pointing data, from measurement set. Creates holography output file.
 
@@ -81,6 +305,9 @@ def extract_holog(
     :param data_column: Determines the data column to pull from the measurement set. Defaults to "CORRECTED_DATA".
     :type data_column: str, optional, ex. DATA, CORRECTED_DATA
 
+    :param time_smoothing_interval: Determines the time smoothing interval, set to the integration time when None.
+    :type time_smoothing_interval: float, optional
+
     :param parallel: Boolean for whether to process in parallel, defaults to False.
     :type parallel: bool, optional
 
@@ -114,7 +341,7 @@ def extract_holog(
             ddi_m: …
         }
 
-    **Example Usage** In this case the pointing file has already been created. In addition the appropriate
+    **Example Usage** In this case the pointing file has already been created. In addition, the appropriate
     data_column value nees to be set for the type of measurement set data you are extracting.
 
     .. parsed-literal::
@@ -172,40 +399,29 @@ def extract_holog(
             }
 
     """
+    # Doing this here allows it to get captured by locals()
+    if holog_name is None:
+        holog_name = get_default_file_name(input_file=ms_name, output_type=".holog.zarr")
+
     extract_holog_params = locals()
 
-    logger = _get_astrohack_logger()
-
-    function_name = inspect.stack()[CURRENT_FUNCTION].function
-
-    ######### Parameter Checking #########
-    extract_holog_params = _check_extract_holog_params(function_name=function_name,
-                                                       extract_holog_params=extract_holog_params)
     input_pars = extract_holog_params.copy()
 
-    _check_if_file_exists(extract_holog_params['ms_name'])
-    _check_if_file_will_be_overwritten(extract_holog_params['holog_name'], extract_holog_params['overwrite'])
+    assert pathlib.Path(extract_holog_params['ms_name']).exists() is True, (
+        logger.error(f'File {extract_holog_params["ms_name"]} does not exists.')
+    )
 
-    if holog_name is None:
-        logger.debug(
-            '[{caller}]: File {file} does not exists. Extracting ...'.format(caller=function_name, file=holog_name))
-
-        holog_name = _remove_suffix(ms_name, '.ms') + '.holog.zarr'
-        extract_holog_params['holog_name'] = holog_name
-
-        logger.debug('[{caller}]: Extracting holog to {output}'.format(caller=function_name, output=holog_name))
+    overwrite_file(extract_holog_params['holog_name'], extract_holog_params['overwrite'])
 
     try:
-        pnt_dict = _load_point_file(extract_holog_params['point_name'])
+        pnt_dict = load_point_file(extract_holog_params['point_name'])
 
     except Exception as error:
-        logger.error('[{function_name}]: Error loading {name}. - {error}'.format(function_name=function_name,
-                                                                                 name=extract_holog_params[
-                                                                                     "point_name"], error=error))
+        logger.error('Error loading {name}. - {error}'.format(name=extract_holog_params["point_name"], error=error))
 
         return None
 
-    ######## Get Spectral Windows ########
+    # Get spectral windows
     ctb = ctables.table(
         os.path.join(extract_holog_params['ms_name'], "DATA_DESCRIPTION"),
         readonly=True,
@@ -218,7 +434,7 @@ def extract_holog(
     ms_ddi = np.arange(len(ddi_spw))
     ctb.close()
 
-    ######## Get Antenna IDs and Names ########
+    # Get antenna IDs and names
     ctb = ctables.table(
         os.path.join(extract_holog_params['ms_name'], "ANTENNA"),
         readonly=True,
@@ -232,7 +448,7 @@ def extract_holog(
 
     ctb.close()
 
-    ######## Get Antenna IDs that are in the main table########
+    # Get antenna IDs in the main table
     ctb = ctables.table(
         extract_holog_params['ms_name'],
         readonly=True,
@@ -249,10 +465,12 @@ def extract_holog(
 
     # Create holog_obs_dict or modify user supplied holog_obs_dict.
     ddi = extract_holog_params['ddi']
+    if isinstance(ddi, int):
+        ddi = [ddi]
 
     # Create holog_obs_dict if not specified
     if holog_obs_dict is None:
-        holog_obs_dict = _create_holog_obs_dict(
+        holog_obs_dict = create_holog_obs_dict(
             pnt_dict,
             extract_holog_params['baseline_average_distance'],
             extract_holog_params['baseline_average_nearest'],
@@ -270,23 +488,6 @@ def extract_holog(
                     if ddi_id not in ddi:
                         del holog_obs_dict[ddi_key]
 
-    # logger.info(f"[{function_name}]: holog_obs_dict: \n%s", pformat(list(holog_obs_dict.values())[0], indent=2,
-    # width=2))
-
-    encoded_obj = json.dumps(holog_obs_dict, cls=NumpyEncoder)
-
-    with open(".holog_obs_dict.json", "w") as outfile:
-        json.dump(encoded_obj, outfile)
-
-    ######## Get Scan and Subscan IDs ########
-    # SDM Tables Short Description (https://drive.google.com/file/d/16a3g0GQxgcO7N_ZabfdtexQ8r2jRbYIS/view)
-    # 2.54 ScanIntent (p. 150)
-    # MAP ANTENNA SURFACE : Holography calibration scan
-
-    # 2.61 SubscanIntent (p. 152)
-    # MIXED : Pointing measurement, some antennas are on-source, some off-source
-    # REFERENCE : reference measurement (used for boresight in holography).
-    # Undefined : ?
 
     ctb = ctables.table(
         os.path.join(extract_holog_params['ms_name'], "STATE"),
@@ -295,16 +496,11 @@ def extract_holog(
         ack=False,
     )
 
-    # scan intent (with subscan intent) is stored in the OBS_MODE column of the STATE subtable.
+    # Scan intent (with subscan intent) is stored in the OBS_MODE column of the STATE sub-table.
     obs_modes = ctb.getcol("OBS_MODE")
     ctb.close()
 
-    scan_intent = "MAP_ANTENNA_SURFACE"
-    state_ids = []
-
-    for i, mode in enumerate(obs_modes):
-        if (scan_intent in mode) and ('REFERENCE' not in mode):
-            state_ids.append(i)
+    state_ids = get_valid_state_ids(obs_modes)
 
     spw_ctb = ctables.table(
         os.path.join(extract_holog_params['ms_name'], "SPECTRAL_WINDOW"),
@@ -361,29 +557,40 @@ def extract_holog(
         extract_holog_params["ddi"] = ddi
         extract_holog_params["chan_setup"] = {}
         extract_holog_params["pol_setup"] = {}
-        extract_holog_params["chan_setup"]["chan_freq"] = spw_ctb.getcol("CHAN_FREQ", startrow=spw_setup_id, nrow=1)[0,
-                                                          :]
-        extract_holog_params["chan_setup"]["chan_width"] = spw_ctb.getcol("CHAN_WIDTH", startrow=spw_setup_id, nrow=1)[
-                                                           0, :]
-        extract_holog_params["chan_setup"]["eff_bw"] = spw_ctb.getcol("EFFECTIVE_BW", startrow=spw_setup_id, nrow=1)[0,
-                                                       :]
-        extract_holog_params["chan_setup"]["ref_freq"] = spw_ctb.getcol("REF_FREQUENCY", startrow=spw_setup_id, nrow=1)[
-            0]
+        extract_holog_params["chan_setup"]["chan_freq"] = \
+            spw_ctb.getcol("CHAN_FREQ", startrow=spw_setup_id, nrow=1)[0, :]
+
+        extract_holog_params["chan_setup"]["chan_width"] = \
+            spw_ctb.getcol("CHAN_WIDTH", startrow=spw_setup_id, nrow=1)[0, :]
+
+        extract_holog_params["chan_setup"]["eff_bw"] = \
+            spw_ctb.getcol("EFFECTIVE_BW", startrow=spw_setup_id, nrow=1)[0, :]
+
+        extract_holog_params["chan_setup"]["ref_freq"] = \
+            spw_ctb.getcol("REF_FREQUENCY", startrow=spw_setup_id, nrow=1)[0]
+
         extract_holog_params["chan_setup"]["total_bw"] = \
             spw_ctb.getcol("TOTAL_BANDWIDTH", startrow=spw_setup_id, nrow=1)[0]
+
         extract_holog_params["pol_setup"]["pol"] = pol_str[
             pol_ctb.getcol("CORR_TYPE", startrow=pol_setup_id, nrow=1)[0, :]]
 
         extract_holog_params["telescope_name"] = obs_ctb.getcol("TELESCOPE_NAME")[0]
 
-        # Loop over all beam_scan_ids, a beam_scan_id can conist out of more than one scan in an ms (this is the case
-        # for the VLA pointed mosiacs).
+        # Loop over all beam_scan_ids, a beam_scan_id can consist of more than one scan in a measurement set (this is
+        # the case for the VLA pointed mosaics).
         for holog_map_key in holog_obs_dict[ddi_name].keys():
 
             if 'map' in holog_map_key:
                 scans = holog_obs_dict[ddi_name][holog_map_key]["scans"]
-
-                logger.info(f"[{function_name}]: Processing ddi: {ddi}, scans: {scans}")
+                if len(scans) > 1:
+                    logger.info("Processing ddi: {ddi}, scans: [{min} ... {max}]".format(
+                        ddi=ddi, min=scans[0], max=scans[-1]
+                    ))
+                else:
+                    logger.info("Processing ddi: {ddi}, scan: {scan}".format(
+                        ddi=ddi, scan=scans
+                    ))
 
                 if len(list(holog_obs_dict[ddi_name][holog_map_key]['ant'].keys())) != 0:
                     map_ant_list = []
@@ -417,17 +624,17 @@ def extract_holog(
 
                     if parallel:
                         delayed_list.append(
-                            dask.delayed(_extract_holog_chunk)(
+                            dask.delayed(process_extract_holog_chunk)(
                                 dask.delayed(extract_holog_params)
                             )
                         )
                     else:
-                        _extract_holog_chunk(extract_holog_params)
+                        process_extract_holog_chunk(extract_holog_params)
 
                     count += 1
 
                 else:
-                    logger.warning(f'[{function_name}]: DDI ' + str(ddi) + ' has no holography data to extract.')
+                    logger.warning("DDI " + str(ddi) + " has no holography data to extract.")
 
     spw_ctb.close()
     pol_ctb.close()
@@ -437,90 +644,49 @@ def extract_holog(
         dask.compute(delayed_list)
 
     if count > 0:
-        logger.info(f"[{function_name}]: Finished processing")
+        logger.info("Finished processing")
+
+        holog_dict = load_holog_file(
+            file=extract_holog_params["holog_name"],
+            dask_load=True,
+            load_pnt_dict=False
+        )
 
-        holog_dict = _load_holog_file(holog_file=extract_holog_params["holog_name"], dask_load=True,
-                                      load_pnt_dict=False)
         extract_holog_params['telescope_name'] = telescope_name
 
-        meta_data = _create_holog_meta_data(
+        meta_data = create_holog_meta_data(
             holog_file=extract_holog_params['holog_name'],
             holog_dict=holog_dict,
             input_params=extract_holog_params.copy()
         )
 
         holog_attr_file = "{name}/{ext}".format(name=extract_holog_params['holog_name'], ext=".holog_attr")
-        _write_meta_data(holog_attr_file, meta_data)
+        write_meta_data(holog_attr_file, meta_data)
+
         holog_attr_file = "{name}/{ext}".format(name=extract_holog_params['holog_name'], ext=".holog_input")
-        _write_meta_data(holog_attr_file, input_pars)
+        write_meta_data(holog_attr_file, input_pars)
+
+        with open(f"{extract_holog_params['holog_name']}/holog_obs_dict.json", "w") as outfile:
+            json.dump(holog_obs_dict, outfile, cls=NumpyEncoder)
 
         holog_mds = AstrohackHologFile(extract_holog_params['holog_name'])
-        holog_mds._open()
+        holog_mds.open()
 
         return holog_mds
 
     else:
-        logger.warning(f"[{function_name}]: No data to process")
+        logger.warning("No data to process")
         return None
 
 
-def _check_extract_holog_params(function_name, extract_holog_params):
-    #### Parameter Checking ####
-    logger = _get_astrohack_logger()
-    parms_passed = True
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'ms_name', [str], default=None)
-
-    base_name = _remove_suffix(extract_holog_params['ms_name'], '.ms')
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'holog_name', [str],
-                                                 default=base_name + '.holog.zarr')
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'point_name', [str], default=None)
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'ddi', [list, int, str],
-                                                 list_acceptable_data_types=[int], default='all')
-
-    # To Do: special function needed to check holog_obs_dict.
-    parm_check = isinstance(extract_holog_params['holog_obs_dict'], dict) or (
-            extract_holog_params['holog_obs_dict'] is None)
-    parms_passed = parms_passed and parm_check
-
-    if not parm_check:
-        logger.error(f'[{function_name}]: Parameter holog_obs_dict must be of type {str(dict)}.')
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'baseline_average_distance',
-                                                 [int, float, str], default='all')
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'baseline_average_nearest',
-                                                 [int, str], default='all')
-
-    if (extract_holog_params['baseline_average_distance'] != 'all') and (
-            extract_holog_params['baseline_average_nearest'] != 'all'):
-        logger.error(
-            f'[{function_name}]: baseline_average_distance and 'f'baseline_average_nearest can not both be specified.')
-
-        parms_passed = False
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'data_column', [str],
-                                                 default='CORRECTED_DATA')
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'parallel', [bool], default=False)
-
-    parms_passed = parms_passed and _check_parms(function_name, extract_holog_params, 'overwrite', [bool],
-                                                 default=False)
-
-    _parm_check_passed(function_name, parms_passed)
-
-    return extract_holog_params
-
-
 def generate_holog_obs_dict(
-        ms_name,
-        point_name,
-        baseline_average_distance='all',
-        baseline_average_nearest='all',
-        parallel=False
-):
+        ms_name: str,
+        point_name: str,
+        baseline_average_distance: str = 'all',
+        baseline_average_nearest: str = 'all',
+        write=True,
+        parallel: bool = False
+) -> HologObsDict:
     """
     Generate holography observation dictionary, from measurement set..
 
@@ -540,6 +706,9 @@ def generate_holog_obs_dict(
     baseline_average_nearest can not be used together.
     :type baseline_average_nearest: int, optional
 
+    :param write: Write file flag.
+    :type point_name: bool, optional
+
     :param point_name: Name of *<point_name>.point.zarr* file to use. 
     :type point_name: str, optional
 
@@ -623,28 +792,10 @@ def generate_holog_obs_dict(
     """
     extract_holog_params = locals()
 
-    logger = _get_astrohack_logger()
+    assert pathlib.Path(ms_name).exists() is True, logger.error(f'File {ms_name} does not exists.')
+    assert pathlib.Path(point_name).exists() is True, logger.error(f'File {point_name} does not exists.')
 
-    function_name = inspect.stack()[CURRENT_FUNCTION].function
-
-    _check_if_file_exists(ms_name)
-    _check_if_file_exists(point_name)
-
-    ######## Get Spectral Windows ########
-    ctb = ctables.table(
-        os.path.join(extract_holog_params['ms_name'], "DATA_DESCRIPTION"),
-        readonly=True,
-        lockoptions={"option": "usernoread"},
-        ack=False,
-    )
-
-    ddi_spw = ctb.getcol("SPECTRAL_WINDOW_ID")
-    ddpol_indexol = ctb.getcol("POLARIZATION_ID")
-    ms_ddi = np.arange(len(ddi_spw))
-
-    ctb.close()
-
-    ######## Get Antenna IDs and Names ########
+    # Get antenna IDs and names
     ctb = ctables.table(
         os.path.join(extract_holog_params['ms_name'], "ANTENNA"),
         readonly=True,
@@ -658,7 +809,7 @@ def generate_holog_obs_dict(
 
     ctb.close()
 
-    ######## Get Antenna IDs that are in the main table########
+    # Get antenna IDs that are in the main table
     ctb = ctables.table(
         extract_holog_params['ms_name'],
         readonly=True,
@@ -673,13 +824,10 @@ def generate_holog_obs_dict(
     ant_names_main = ant_names[ant_id_main]
     ctb.close()
 
-    # Create holog_obs_dict or modify user supplied holog_obs_dict.
-    # ddi = extract_holog_params['ddi']
-
     pnt_mds = AstrohackPointFile(extract_holog_params['point_name'])
-    pnt_mds._open()
+    pnt_mds.open()
 
-    holog_obs_dict = _create_holog_obs_dict(
+    holog_obs_dict = create_holog_obs_dict(
         pnt_mds,
         extract_holog_params['baseline_average_distance'],
         extract_holog_params['baseline_average_nearest'],
@@ -689,205 +837,117 @@ def generate_holog_obs_dict(
         write_distance_matrix=True
     )
 
-    # From the generated holog_obs_dict subselect user supplied ddis.
-    # if ddi != 'all':
-    #    holog_obs_dict_keys = list(holog_obs_dict.keys())
-    #    for ddi_key in holog_obs_dict_keys:
-    #        if 'ddi' in ddi_key:
-    #            ddi_id = int(ddi_key.replace('ddi_', ''))
-    #            if ddi_id not in ddi:
-    #                del holog_obs_dict[ddi_key]
-
     encoded_obj = json.dumps(holog_obs_dict, cls=NumpyEncoder)
 
-    with open(".holog_obs_dict.json", "w") as outfile:
-        json.dump(encoded_obj, outfile)
+    if write:
+        with open("holog_obs_dict.json", "w") as outfile:
+            outfile.write(encoded_obj)
 
     return HologObsDict(json.loads(encoded_obj))
 
 
-class HologObsDict(dict):
-    """
-      ddi --> map --> ant, scan
-                      |
-                      o--> map: [reference, ...]
+def get_number_of_parameters(holog_obs_dict: HologObsDict) -> Tuple[int, int, int, int]:
+    scan_list = []
+    ant_list = []
+    baseline_list = []
+
+    for ddi in holog_obs_dict.keys():
+        for mapping in holog_obs_dict[ddi].keys():
+            scan_list.append(len(holog_obs_dict[ddi][mapping]["scans"]))
+            ant_list.append(len(holog_obs_dict[ddi][mapping]["ant"].keys()))
+
+            for ant in holog_obs_dict[ddi][mapping]["ant"].keys():
+                baseline_list.append(len(holog_obs_dict[ddi][mapping]["ant"][ant]))
+
+    n_ddi = len(holog_obs_dict.keys())
+    n_scans = max(scan_list)
+    n_ant = max(ant_list)
+    n_baseline = max(baseline_list)
+
+    return n_ddi, n_scans, n_ant, n_baseline
+
+
+def model_memory_usage(
+        ms_name: str,
+        holog_obs_dict: HologObsDict = None
+) -> int:
+    """ Determine the approximate memory usage per core of a given measurement file.
+
+        :param ms_name: Measurement file name
+        :type ms_name: str
+
+        :param holog_obs_dict: Holography observations dictionary.
+        :type holog_obs_dict: HologObsDict, optional
+
+        :return: Memory per core
+        :rtype: int
     """
 
-    def __init__(self, obj):
-        super().__init__(obj)
-        self.logger = _get_astrohack_logger()
+    # Get holog observations dictionary
+    if holog_obs_dict is None:
+        extract_pointing(
+            ms_name=ms_name,
+            point_name="temporary.pointing.zarr",
+            parallel=False,
+            overwrite=True,
+        )
 
-    def __getitem__(self, key):
-        return super().__getitem__(key)
+        holog_obs_dict = generate_holog_obs_dict(
+            ms_name=ms_name,
+            point_name="temporary.pointing.zarr",
+            baseline_average_distance='all',
+            baseline_average_nearest='all',
+            parallel=False
+        )
 
-    def __setitem__(self, key, value):
-        return super().__setitem__(key, value)
+        shutil.rmtree("temporary.pointing.zarr")
 
-    def print(self, style="static"):
-        if style == "dynamic":
-            return astrohack.dio.inspect_holog_obs_dict(self, style="dynamic")
+    # Get number of each parameter
+    n_ddi, n_scans, n_ant, n_baseline = get_number_of_parameters(holog_obs_dict)
 
-        else:
-            return astrohack.dio.inspect_holog_obs_dict(self, style="static")
+    # Get model file
+    if not pathlib.Path("model").exists():
+        os.mkdir("model")
 
-    def select(self, key, value, inplace=False, **kwargs):
+    toolviper.utils.data.download('heuristic_model', folder="model")
 
-        if inplace:
-            obs_dict = self
+    with open("model/elastic.model", "rb") as model_file:
+        model = pickle.load(model_file)
 
-        else:
-            obs_dict = HologObsDict(copy.deepcopy(self))
+    memory_per_core = math.ceil(model.predict([[n_ddi, n_scans, n_ant, n_baseline]])[0])
 
-        if key == "ddi":
-            return self._select_ddi(value, obs_dict=obs_dict)
+    cores = multiprocessing.cpu_count()
+    memory_available = round((psutil.virtual_memory().available / (1024 ** 2)))
+    memory_limit = round((psutil.virtual_memory().total / (1024 ** 2)))
 
-        elif key == "map":
-            return self._select_ddi(value, obs_dict=obs_dict)
+    table = Table(
+        title="System Info",
+        caption="Available memory: represents the system memory available without going into swap"
+    )
 
-        elif key == "antenna":
-            return self._select_antenna(value, obs_dict=obs_dict)
+    table.add_column("N-cores", justify="right", style="blue", no_wrap=True)
+    table.add_column("Available memory (MB)", style="magenta")
+    table.add_column("Total memory (MB)", style="cyan")
+    table.add_column("Suggested memory per core (MB)", justify="right", style="green")
 
-        elif key == "scan":
-            return self._select_scan(value, obs_dict=obs_dict)
+    table.add_row(str(cores), str(memory_available), str(memory_limit), str(memory_per_core))
 
-        elif key == "baseline":
-            if "reference" in kwargs.keys():
-                return self._select_baseline(
-                    value,
-                    n_baselines=None,
-                    reference=kwargs["reference"],
-                    obs_dict=obs_dict
-                )
+    console = Console()
+    console.print(table)
 
-            elif "n_baselines" in kwargs.keys():
-                return self._select_baseline(
-                    value,
-                    n_baselines=kwargs["n_baselines"],
-                    reference=None,
-                    obs_dict=obs_dict
-                )
+    # Make prediction of memory per core in MB
+    return memory_per_core
 
-            else:
-                self.logger.error("Must specify a list of reference antennas for this option.")
-        else:
-            self.logger.error("Valid key not found: {key}".format(key=key))
-            return {}
 
-    @staticmethod
-    def get_nearest_baselines(antenna, n_baselines=None, path_to_matrix=None):
-        logger = _get_astrohack_logger()
-        import pandas as pd
+def _convert_ant_name_to_id(ant_list, ant_names):
+    """_summary_
 
-        if path_to_matrix is None:
-            path_to_matrix = os.getcwd() + "/.baseline_distance_matrix.csv"
+  Args:
+      ant_list (_type_): _description_
+      ant_names (_type_): _description_
 
-        if not os.path.exists(path_to_matrix):
-            logger.error("Unable to find baseline distance matrix in: {path}".format(path=path_to_matrix))
+  Returns:
+      _type_: _description_
+  """
 
-        df_matrix = pd.read_csv(path_to_matrix, sep="\t", index_col=0)
-
-        # Skip the first index because it is a self distance
-        if n_baselines is None:
-            return df_matrix[antenna].sort_values(ascending=True).index[1:].values.tolist()
-
-        return df_matrix[antenna].sort_values(ascending=True).index[1:n_baselines].values.tolist()
-
-    @staticmethod
-    def _select_ddi(value, obs_dict):
-        convert = lambda x: "ddi_" + str(x)
-
-        if not isinstance(value, list):
-            value = [value]
-
-        value = list(map(convert, value))
-        ddi_list = list(obs_dict.keys())
-
-        for ddi in ddi_list:
-            if ddi not in value:
-                obs_dict.pop(ddi)
-
-        return obs_dict
-
-    @staticmethod
-    def _select_map(value, obs_dict):
-        convert = lambda x: "map_" + str(x)
-
-        if not isinstance(value, list):
-            value = [value]
-
-        value = list(map(convert, value))
-        ddi_list = list(obs_dict.keys())
-
-        for ddi in ddi_list:
-            map_list = list(obs_dict[ddi].keys())
-            for mp in map_list:
-                if mp not in value:
-                    obs_dict[ddi].pop(mp)
-
-        return obs_dict
-
-    @staticmethod
-    def _select_antenna(value, obs_dict):
-        if not isinstance(value, list):
-            value = [value]
-
-        ddi_list = list(obs_dict.keys())
-
-        for ddi in ddi_list:
-            map_list = list(obs_dict[ddi].keys())
-            for mp in map_list:
-                ant_list = list(obs_dict[ddi][mp]["ant"].keys())
-                for ant in ant_list:
-                    if ant not in value:
-                        obs_dict[ddi][mp]["ant"].pop(ant)
-
-        return obs_dict
-
-    @staticmethod
-    def _select_scan(value, obs_dict):
-        if not isinstance(value, list):
-            value = [value]
-
-        ddi_list = list(obs_dict.keys())
-
-        for ddi in ddi_list:
-            map_list = list(obs_dict[ddi].keys())
-            for mp in map_list:
-                obs_dict[ddi][mp]["scans"] = value
-
-        return obs_dict
-
-    @staticmethod
-    def _select_baseline(value, n_baselines, obs_dict, reference=None):
-        if reference is not None:
-            if not isinstance(reference, list):
-                reference = [reference]
-
-        ddi_list = list(obs_dict.keys())
-
-        for ddi in ddi_list:
-            map_list = list(obs_dict[ddi].keys())
-            for mp in map_list:
-                ant_list = list(obs_dict[ddi][mp]["ant"].keys())
-                for ant in ant_list:
-                    if ant not in value:
-                        obs_dict[ddi][mp]["ant"].pop(ant)
-                        continue
-
-                    if reference is None and n_baselines is not None:
-                        reference_antennas = obs_dict[ddi][mp]["ant"][ant]
-
-                        if n_baselines > len(reference_antennas):
-                            n_baselines = len(reference_antennas)
-
-                        sorted_antennas = np.array(obs_dict.get_nearest_baselines(antenna=ant))
-
-                        values, i, j = np.intersect1d(reference_antennas, sorted_antennas, return_indices=True)
-                        index = np.sort(j)
-
-                        obs_dict[ddi][mp]["ant"][ant] = sorted_antennas[index][:n_baselines]
-
-                    else:
-                        obs_dict[ddi][mp]["ant"][ant] = reference
-
-        return obs_dict
+    return np.nonzero(np.in1d(ant_list, ant_names))[0]
